@@ -1,23 +1,14 @@
 use crate::arpx_job::task::{task, Task};
 use crate::prelude::*;
-use std::ops::Deref;
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Job {
     pub tasks: Vec<Task>,
 }
 
-impl Deref for Job {
-    type Target = Vec<Task>;
-
-    fn deref(&self) -> &Self::Target {
-        &self.tasks
-    }
-}
-
 #[must_use]
 pub fn job<'a>() -> impl Parser<'a, Job> {
-    n(whitespace_wrap(task()), 1..).map(|tasks| Job { tasks })
+    n(whitespace_wrap(task()), 0..).map(|tasks| Job { tasks })
 }
 
 #[cfg(test)]
