@@ -3,8 +3,8 @@ use crate::prelude::*;
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct Process {
     pub name: String,
-    pub onfail: String,
-    pub onsucceed: String,
+    pub onfail: Option<Box<Process>>,
+    pub onsucceed: Option<Box<Process>>,
     pub silent: bool,
 }
 
@@ -76,25 +76,30 @@ fn process_name(input: &str) -> ParseResult<String> {
     Ok((&input[next_index..], matched))
 }
 
-fn process_predicate<'a>() -> impl Parser<'a, (String, String)> {
-    pair(
-        optional(onsucceed()).map(|result| match result {
-            Some(input) => input,
-            None => String::new(),
-        }),
-        optional(onfail()).map(|result| match result {
-            Some(input) => input,
-            None => String::new(),
-        }),
-    )
+fn process_predicate<'a>() -> impl Parser<'a, (Option<Box<Process>>, Option<Box<Process>>)> {
+    pair(optional(onsucceed()), optional(onfail()))
 }
 
-fn onsucceed<'a>() -> impl Parser<'a, String> {
-    right(whitespace_wrap(literal("?")), process_name)
+fn onsucceed<'a>() -> impl Parser<'a, Box<Process>> {
+    right(whitespace_wrap(literal("?")), process_name).map(|name| {
+        Box::new(Process {
+            name,
+            onsucceed: None,
+            onfail: None,
+            silent: false,
+        })
+    })
 }
 
-fn onfail<'a>() -> impl Parser<'a, String> {
-    right(whitespace_wrap(literal(":")), process_name)
+fn onfail<'a>() -> impl Parser<'a, Box<Process>> {
+    right(whitespace_wrap(literal(":")), process_name).map(|name| {
+        Box::new(Process {
+            name,
+            onsucceed: None,
+            onfail: None,
+            silent: false,
+        })
+    })
 }
 
 #[cfg(test)]
@@ -108,8 +113,8 @@ mod tests {
 
         let expected = vec![Process {
             name: "loop1".to_string(),
-            onsucceed: String::new(),
-            onfail: String::new(),
+            onsucceed: None,
+            onfail: None,
             silent: false,
         }];
 
@@ -123,8 +128,8 @@ mod tests {
 
         let expected = vec![Process {
             name: "loop1".to_string(),
-            onsucceed: String::new(),
-            onfail: String::new(),
+            onsucceed: None,
+            onfail: None,
             silent: true,
         }];
 
@@ -138,8 +143,13 @@ mod tests {
 
         let expected = vec![Process {
             name: "loop1".to_string(),
-            onsucceed: "loop2".to_string(),
-            onfail: String::new(),
+            onsucceed: Some(Box::new(Process {
+                name: "loop2".to_string(),
+                onsucceed: None,
+                onfail: None,
+                silent: false,
+            })),
+            onfail: None,
             silent: false,
         }];
 
@@ -153,8 +163,13 @@ mod tests {
 
         let expected = vec![Process {
             name: "loop1".to_string(),
-            onsucceed: String::new(),
-            onfail: "loop3".to_string(),
+            onsucceed: None,
+            onfail: Some(Box::new(Process {
+                name: "loop3".to_string(),
+                onsucceed: None,
+                onfail: None,
+                silent: false,
+            })),
             silent: false,
         }];
 
@@ -168,8 +183,18 @@ mod tests {
 
         let expected = vec![Process {
             name: "loop1".to_string(),
-            onsucceed: "loop2".to_string(),
-            onfail: "loop3".to_string(),
+            onsucceed: Some(Box::new(Process {
+                name: "loop2".to_string(),
+                onsucceed: None,
+                onfail: None,
+                silent: false,
+            })),
+            onfail: Some(Box::new(Process {
+                name: "loop3".to_string(),
+                onsucceed: None,
+                onfail: None,
+                silent: false,
+            })),
             silent: false,
         }];
 
@@ -183,8 +208,18 @@ mod tests {
 
         let expected = vec![Process {
             name: "loop1".to_string(),
-            onsucceed: "loop2".to_string(),
-            onfail: "loop3".to_string(),
+            onsucceed: Some(Box::new(Process {
+                name: "loop2".to_string(),
+                onsucceed: None,
+                onfail: None,
+                silent: false,
+            })),
+            onfail: Some(Box::new(Process {
+                name: "loop3".to_string(),
+                onsucceed: None,
+                onfail: None,
+                silent: false,
+            })),
             silent: true,
         }];
 
@@ -205,20 +240,50 @@ mod tests {
         let expected = vec![
             Process {
                 name: "loop1".to_string(),
-                onsucceed: "loop2".to_string(),
-                onfail: "loop3".to_string(),
+                onsucceed: Some(Box::new(Process {
+                    name: "loop2".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
+                onfail: Some(Box::new(Process {
+                    name: "loop3".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
                 silent: false,
             },
             Process {
                 name: "loop2".to_string(),
-                onsucceed: "loop3".to_string(),
-                onfail: "loop4".to_string(),
+                onsucceed: Some(Box::new(Process {
+                    name: "loop3".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
+                onfail: Some(Box::new(Process {
+                    name: "loop4".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
                 silent: false,
             },
             Process {
                 name: "loop3".to_string(),
-                onsucceed: "loop4".to_string(),
-                onfail: "loop5".to_string(),
+                onsucceed: Some(Box::new(Process {
+                    name: "loop4".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
+                onfail: Some(Box::new(Process {
+                    name: "loop5".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
                 silent: true,
             },
         ];
@@ -235,14 +300,29 @@ mod tests {
         let expected_2 = vec![
             Process {
                 name: "loop1".to_string(),
-                onsucceed: "loop2".to_string(),
-                onfail: "loop3".to_string(),
+                onsucceed: Some(Box::new(Process {
+                    name: "loop2".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
+                onfail: Some(Box::new(Process {
+                    name: "loop3".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
                 silent: false,
             },
             Process {
                 name: "loop2".to_string(),
-                onsucceed: "loop3".to_string(),
-                onfail: String::new(),
+                onsucceed: Some(Box::new(Process {
+                    name: "loop3".to_string(),
+                    onsucceed: None,
+                    onfail: None,
+                    silent: false,
+                })),
+                onfail: None,
                 silent: false,
             },
         ];
